* Motorola Flash Innovation 2003 Contest Project F193
*
* datavid.asm  	for 68HC908QY4		Copyright 2003 Jed Margolin
************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
************************************************************************
*
* Real-time Clock, Reads and displays adc0 and adc1 numerically in Video 
*
* Hardware: "NTSC Video (With Data Inputs)"
*
* adc0 and adc1 are read and displayed once in video every frame (29.969 Hz)
*
* QY4 Clock is 28.63636 (8 x chroma) to produce a bus clock of 7.15909 MHz 
* which is twice Chroma (3.579545 MHz).  Bus clock is 2 x chroma.
*
* HSync is 15.734 KHz. (chroma x 2/455 as per NTSC spec)
*
* The Realtime clock is corrected for the 15734/525 = 29.969 Hz Frame Rate
*
#pagelength 65t
#pagewidth 124t
$base 10t

NVL  	  EQU	253	; 10 for testing VBI, 253 for real sync
TIMECAL	  EQU	984	; 

$INCLUDE 'qy4equ.h'    	; Equates for MC68HC908QY4 MCU
*=====================================================
	ORG	RAMstart

vcount		RMB    1
vline		RMB    1
vlfunction	RMB    2
chline		RMB    1
ch0address	RMB    2	; character 0 address
ch1address	RMB    2	; character 1 address
ch2address	RMB    2	; character 2 address
ch3address	RMB    2	; character 3 address
ch4address	RMB    2	; character 4 address
ch5address	RMB    2	; character 5 address
ch6address	RMB    2	; character 6 address
ch7address	RMB    2	; character 7 address
ch8address	RMB    2	; character 8 address
clnaddress	RMB    2	; colon address
maddress	RMB    2	; am/pm address
chtimes		RMB    1	; number of lines to repeat/character line
time:		RMB    3	; hours:minutes:seconds
timecount:	RMB    1
clockflag:	RMB    1
clockcal:	RMB    2   	; calibration factor to correct for frame rate
ch0adstor	RMB    2	; character 0 address storage
ch1adstor	RMB    2	; character 1 address storage
ch2adstor	RMB    2	; character 2 address storage
ch3adstor	RMB    2	; character 3 address storage
ch4adstor	RMB    2	; character 4 address storage
ch5adstor	RMB    2	; character 5 address storage
ch6adstor	RMB    2	; character 6 address storage
ch7adstor	RMB    2	; character 7 address storage
ch8adstor	RMB    2	; character 8 address storage
madstor		RMB    2	; A or M character address storage
timer_vec	RMB    2   	; Timer Interrupt Redirection Address
sw1		RMB    3	; increment switch processing
sw2		RMB    3	; increment switch processing
bcd		RMB    2	; bcd data
bin		RMB    1	; binay data
*====================================================
* Video Signals
CSYNC	 	EQU 	0	; Composite Sync
VIDEO	 	EQU 	1	; 1 = On
nHVIDEO	 	EQU 	2	; 0 = Half Bright 
FIELD		EQU 	3	; 1 = Field 1
VBLANK 		EQU 	4	; V Blanking
IncTime		EQU 	5	; 0 = Increment Time
DecTime		EQU 	6	; 0 = Decrement Time
*====================================================
	ORG	$EE00

* First initialize any control registers and variables
START:  LDHX	#$0100		; locate the stack
	TXS

	RSP

	MOV	#$01,CONFIG1	; disable COP
*------------------------------------------------------------------------
* Clock Source
* External Crystal
	CLR	PORTA	
	BSET	4,DDRA		; PA4 output
	BSET	PA4,PORTA	; to help get crystal started
	MOV	#$3C,PAPUP	; Leave adc inputs floating

	MOV	#$19,CONFIG2	; reset pin active, external crystal

	BSET	1,OSCSTAT
*------------------------------------------------------------------------
* Initialize I/O Ports
	CLR	PORTB
	CLR	DDRB		; Inputs
	MOV	#$FF,PBPUP	; Enable Port B Pullups 

	MOV	#$1F,PORTB	; HVID is off
	MOV	#$1F,DDRB

	LDHX	#$FF
	STHX	sw1
	STHX	sw2
	CLR	sw1+2
	CLR	sw2+2
	CLR	clockflag

	MOV	#$12,bcd		; adc default
	MOV	#$34,bcd+1

* Initialize ADC 
	MOV	#$60,ADICLK	; 7.159 MHz/8 = 894 KHz
	NOP
	MOV	#$00,ADSCR	; AD Channel 0
*	MOV	#$01,ADSCR	; AD Channel 1

	BSET	ADCO,ADSCR	; continuous conversion
*------------------------------------------------------------------------
	BRA	MAIN
*------------------------------------------------------------------------
* NTSC Sync:
*
* Bus clock is twice chroma (3.579545 * 2 = 7.15909), which is why the 
* clock is 28.63636 (28.63636/4 = 7.15909)  
*
* Horizontal frequency is defined as chroma * 2/455 = 15.734263 KHz
* Timer Interrupts can be set as an integer number of bus clocks, so it
* is set to 455 cycles.
*
* Everything is done at the interrupt level.
*
* The main line code is just NOPs because they are single cycle instructions.
*
* Some things are done at twice the horizontal rate and require cycle 
*   counting.
*
* Having 455 CPU cyles per Horizontal Line makes it impossible to divide
*   the H Line exactly in half for EQ and VSync pulses. 
*
* Interlacing is done as per NTSC spec by moving every other VBI 1/2H early
*
MAIN:	MOV	#NVL,vcount  ; number of active video lines/field
			     ; 10 for testing, 253 for real sync

	MOV	#9,vline     ; number of lines in Vertical Blanking Interval
	MOV	#8,chline

	LDHX	#$0023		; default time
	STHX	time
	MOV	#$45,time+2

* Initialize display character addresses
	JSR	Get_CH		; get character addresses

	CLR	chtimes
	MOV	#30,timecount	; 30 fields/sec for the clock

	LDHX	#TIMECAL	; clock correction for 15.734 KHz
	STHX	clockcal

	LDHX	#TIRQ1		; Timer Overflow Vector: Real Horizontal Sync
	STHX	timer_vec

	LDHX	#vlf1		; the first video line function
	STHX	vlfunction

	CLRA
	JSR	TIRQ_Init	; Initialize timer irqs

	JMP	NOPS		; Everything gets done in the timer interrupt 
*=============================================================================
* Initialize Timer Interrupts
TIRQ_Init:

* Timer counter, must write to high byte first, 455 -> 0-454
	MOV	#$01,TMODH		; 454 = $01C6
	MOV	#$C6,TMODL

	MOV	#$00,TSC		; PS2, PS1, PS0; TSTOP = 0

	BSET	TOIE,TSC	; enable interrupts

	CLI

	RTS
*==========================================================================
* Timebase 						($021C)
* Jump to Address in User's Vector Table
TIM_INT:
*        PSHX		; we don't care about trashing XRegs
*        PSHH

	LDHX	timer_vec
	JMP	,X
*==========================================================================
UNUSED: RTI
*==========================================================================
***************************************************************************
* Timer Overflow Interrupt (TIM_OF_VEC)
* Active Video: Field 2 begins on Line 271 1/2, ends on Line 525
* Real Horizontal Sync
TIRQ1:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq1a:	DBNZX	tq1a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq1c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ2	 ; Timer Overflow Vector: Start Field 1 VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	BSET	FIELD,PORTB	; Field 1 flag

	BRA	TIRQRET

* Video is still active. We have about 50 us to do something else
tq1c:	JSR	Do_Video
	BRA	TIRQRET
*======================================================================
NOPORG	EQU	$FC30
NOPSHI	EQU	{NOPORG > 8}	; high byte of .ORG for nops.h
NOPSLO	EQU	NOPORG&$FF	; low byte of .ORG for nops.h
*---------
* RTI to beginning of NOPS to make sure an interrupt does not occur during
* the 2-cycle JMP instruction.
TIRQRET:
	TSX			; 2	SP to H:X

	LDA	#NOPSHI		; 2	NOPS High Address
	STA	3,X		; 4

	LDA	#NOPSLO		; 2	NOPS Low Address
	STA	4,X		; 4

	RTI			; 7	(21)
*==========================================================================
* Field 1 VBI: Leading Equalization Pulses
TIRQ2:	

* Leading Equalization Pulses
* We need to do two equalization pulses per line
*
* we need to align it with HSync
* Line 1
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 2
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 3
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Field 1 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
* Line 4
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* We need to do two VSync pulses per line
* Line 5
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* We need to do two VSync pulses per line
* Line 6
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
* Line 7
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* We need to do two equalization pulses per line
* Line 8
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* We need to do two equalization pulses per line
* Line 9
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

	LDHX	#TIRQ3	    	; Timer Overflow Vector: active video
	STHX	timer_vec
	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#vlf1		; the first video line function
	STHX	vlfunction

	JMP	TIRQRET
*======================
* Active Video: Field 1 begins on Line 10, ends on Line 262 1/2
* Real Horizontal Sync
TIRQ3:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq3a:	DBNZX	tq3a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq3c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ4	     ; Timer Overflow Vector: Start VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	JMP	TIRQRET

* Video is still active
* We have about 50 us to do something else
tq3c:	JSR	Do_Video
	JMP	TIRQRET
*========================================================================
* Active Video: end of Field 1
* Start Field 2 VBI 
* Do 1/2 Horizontal line and 1 Equalizing pulse
* Real Horizontal Sync
TIRQ4:	

* Line 262
*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#8		; [2]
tq4a:	DBNZX	tq4a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Bring cycle count from CSYNC to 228 (455/2 or so)
	LDX	#61
tq4b:	DBNZX	tq4b
	NOP
	BCLR	FIELD,PORTB	; [4] Field 1 flag
*---------
* Do 1 Equalizing Pulse
* Do the pulse and get ready for the next step
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2

	LDHX	#TIRQ5	    ; Timer Overflow Vector: active video
	STHX	timer_vec

	JMP	TIRQRET
*======================
* Field 2 VBI: Equalization Pulses
TIRQ5:	
* Line 263
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 264
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 455 cycles
* Line 265
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
	NOP			; [1]

* end at 228 cycles (6 EQ pulses)
* Field 2 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 266
* end at 455 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

* end at 228 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 267
* end at 455 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 455 cycles
* Line 268
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1
	NOP			; [1]

* end at 228 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 269
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 270
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	
	NOP			; [1]

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 455 cycles
* Line 271
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

* Start of Field 2 Video
	LDX	#7		; [2]	add 4+4+4
tq5a:	DBNZX	tq5a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#5	  ; [2]		57
tq5b:	DBNZX	tq5b	  ; [3]
	NOP		  ; [1]

* End of trailing equalization pulses
* Line 271 1/2 end early to give cycles to doing the clock
* This only affects the VBLANK signal
	BCLR	VBLANK,PORTB	; oscilloscope trigger low
	MOV	#9,vline
	JSR	Do_Time		; do the clock 
	JSR	Get_CH		; get character addresses into display list
*---------
* Start Field 2 Active Video, which ends in an integer number of H lines
* and begins Field 1 VBI
	LDHX	#TIRQ1		; Timer Overflow Vector
	STHX	timer_vec

	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#vlf1		; the first video line function
	STHX	vlfunction

	JMP	TIRQRET
*============================================================================
* Equalization Pulse
EQ1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#2		; [2]
eq1a:	DBNZX	eq1a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#66		; [2]
eq1b:	DBNZX	eq1b	   	; [3]
	NOP

	RTS			; [4]
*==========================================================================
* Equalization Pulse
EQ2:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#1		; [2]
eq2a:	DBNZX	eq2a		; [3]
	NOP			; [1]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

	RTS			; [4]
*==========================================================================
* VSync pulse
VS1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#61
vs1a:	DBNZX	vs1a	   	; [3]	
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off
	
	LDX	#7		; [3]  {7}
vs1b:	DBNZX	vs1b	   	; [3]	
	NOP			; [1]
	RTS
*==========================================================================
***************************************************************************
*==========================================================================
* Wait 1000 us (for testing the clock frequency)
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 7.159 MHz, we need 1000*7.159 = 7159 cycles
* xval = (time*BClock -19)/8
* time = (xval*8 +19)/BClock
* For BClock = 7.159 MHz, xval = 893
* At 2.4576 MHz, time for xval = 893 is 2915 us

*			JSR		4
WAIT1ms:
        PSHX		;		2
        PSHH		;		2

	LDHX	#893	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	PULH		;		2
	PULX		;		2
	RTS		;		4
*=============================================================================
TEST1:	BSET	CSYNC,PORTB
	JSR	WAIT1ms
	BCLR	CSYNC,PORTB
	JSR	WAIT1ms
	BRA	TEST1
*=============================================================================
CHTABLE	FDB	ch_0,ch_1,ch_2,ch_3,ch_4,ch_5,ch_6,ch_7,ch_8,ch_9

* Video Line Functions
Do_Video:
	BCLR	nHVIDEO,PORTB	; half bright
	LDA	vline
	INC	vline		; for next time

	LDHX	vlfunction
	JMP	,X

* vlf1 - line 60: set up group 1 characters
* vlf2 - do group 1 characters 
*
* vlf3 - line 110: set up group 2 characters
* vlf4 - do group 2 characters 
*
* vlf5 - read the switches and set the clock 
*
* vlf6 - line 160: set up ADC0 data characters, part 1
* vlf7 - set up ADC0 data characters, part 2
* vlf8 - do ADC0 characters 
*
* vlf9 - line 210: set up ADC1 data characters, part 1
* vlf10 - set up ADC1 data characters, part 2
* vlf11 - do ADC1 characters 
*
* vlf12 - the end
*-------------------------------------------------------------
vlf1:
* ACC contains vline number
* Group 1 characters
* Set up characters for next line
	CMP	#50		; start characters at line 60
	BNE	vlf1out		; do characters?

	MOV	#8,chline

*	LDHX	#ch_0		; for testing
	LDHX	ch0adstor
	STHX	ch0address

*	LDHX	#ch_1		; for testing
	LDHX	ch1adstor
	STHX	ch1address

*	LDHX	#ch_2		; for testing
	LDHX	ch2adstor
	STHX	ch2address

*	LDHX	#ch_3		; for testing
	LDHX	ch3adstor
	STHX	ch3address

	LDHX	#ch_cln		; for testing
	STHX	clnaddress

	MOV	#4,chtimes

	LDHX	#vlf2			; start it and display it next time
	STHX	vlfunction

vlf1out:
	RTS
*-------------------------------------------------------------
* Display the group 1 characters
vlf2:	LDA	chline
	BNE	vlf2a		; still going

	LDHX	#vlf3		; next group
	STHX	vlfunction
	BRA	vlf2out

* do the characters
* Wait a little
vlf2a:	LDX	#6		; [2]
vlf2b:	DBNZX	vlf2b  		; [3]	

	LDHX	ch0address
	JSR	,X

	LDHX	ch1address
	JSR	,X

	LDHX	clnaddress
	JSR	,X

	LDHX	ch2address
	JSR	,X

	LDHX	ch3address
	JSR	,X

	DEC	chtimes
	BNE	vlf2out		; repeat the line

	DEC	chline		; next line

	LDHX	ch0address
	AIX	#19		; a line takes 19 bytes
	STHX	ch0address

	LDHX	ch1address
	AIX	#19		; a line takes 19 bytes
	STHX	ch1address

	LDHX	ch2address
	AIX	#19		; a line takes 19 bytes
	STHX	ch2address

	LDHX	ch3address
	AIX	#19		; a line takes 19 bytes
	STHX	ch3address

	LDHX	clnaddress
	AIX	#19		; a line takes 19 bytes
	STHX	clnaddress

	MOV	#4,chtimes

vlf2out:
	RTS
*-------------------------------------------------------------
vlf3:
* ACC contains vline number
* Group 2 characters
* Set up characters for next line
	CMP	#100		; group 2 setup characters?
	BNE	vlf3out		; do characters?

	MOV	#8,chline

	LDHX	#ch_cln
	STHX	clnaddress

	LDHX	ch4adstor
	STHX	ch4address

	LDHX	ch5adstor
	STHX	ch5address

	LDHX	madstor		; A or M
	STHX	maddress

	MOV	#4,chtimes

	LDHX	#vlf4			; start it and display it next time
	STHX	vlfunction

vlf3out:
	RTS
*-------------------------------------------------------------
* Display the group 2 characters
vlf4:	LDA	chline
	BNE	vlf4a		; still going

	LDHX	#vlf5		; next group
	STHX	vlfunction
	BRA	vlf4out

* do the characters
* Wait a little
vlf4a:	LDX	#20		; [2]
vlf4b:	DBNZX	vlf4b  		; [3]	

	LDHX	clnaddress
	JSR	,X

	LDHX	ch4address
	JSR	,X

	LDHX	ch5address
	JSR	,X

	LDX	#4		; [2]
vlf4c:	DBNZX	vlf4c  		; [3]	

	LDHX	maddress
	JSR	,X

	DEC	chtimes
	BNE	vlf4out		; repeat the line

	DEC	chline		; next line

	LDHX	clnaddress
	AIX	#19		; a line takes 19 bytes
	STHX	clnaddress

	LDHX	ch4address
	AIX	#19		; a line takes 19 bytes
	STHX	ch4address

	LDHX	ch5address
	AIX	#19		; a line takes 19 bytes
	STHX	ch5address

	LDHX	maddress
	AIX	#19		; a line takes 19 bytes
	STHX	maddress

	MOV	#4,chtimes

vlf4out:
	RTS
*-------------------------------------------------------------
* We got here because we are done with displaying characters
* We can use the lines for something else
* ACC contains vline
vlf5:	BRSET	1,clockflag,dsw1	; restricts it to 1 field
	BCLR	1,clockflag
	BRA	vlf5out
*-------------------
* Do the switches
dsw1:	LDA	sw1
	BRSET	IncTime,PORTB,dsw1a  ; sets the carry to the state of switch
dsw1a:	ROLA			; shifts Carry into sw1
	STA	sw1
	AND	#$07
	BEQ	dsw1b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw2		; next switch
	STA	sw1+1		; debounced switch = off
	CLR	sw1+2		; reset the switch duration counter
	BRA	dsw2		; switch is not pressed

* switch is pressed
dsw1b:	LDX	sw1+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw1f		; short time

* switch has been pressed for a long time
* increment minutes
	CLR	sw1+2		; reset the duration counter
	JSR	Inc_Hrs
	BRA	dsw3		; only 1 switch at a time

dsw1f:	STX	sw1+2		; save the switch duration counter
	COMA
	EOR	sw1+1		; previous switch state
	AND	#$07
	BNE	dsw2		; same as last time

	CLR	sw1+1		; debounced switch = on
	
* we had a transition from off-to-on
* switch press positive transition
* switch has been pressed for a short time
* increment minutes
	LDA	#$01
	BSR	Inc_Mins
	BRA	dsw3
*-----
* next switch
dsw2:	LDA	sw2
	BRSET	DecTime,PORTB,dsw2a  ; sets the carry to the state of switch
dsw2a:	ROLA			     ; shifts Carry into sw2
	STA	sw2
	AND	#$07
	BEQ	dsw2b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw3
	STA	sw2+1		; debounced switch = off
	CLR	sw2+2		; reset the switch duration counter
	BRA	dsw3		; switch is not pressed

* switch is pressed
dsw2b:	LDX	sw2+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw2f		; short time

* switch has been pressed for a long time
* decrement minutes
	CLR	sw2+2		; reset the duration counter
	JSR	Dec_Hrs
	BRA	dsw3

dsw2f:	STX	sw2+2
	COMA
	EOR	sw2+1		; previous switch state
	AND	#$07
	BNE	dsw3		; same as last time

	CLR	sw2+1		; debounced switch = on

* switch has been pressed for a short time
* increment minutes
* decrement minutes
	LDA	#$99		; add 100 - 1
	BSR	Dec_Mins
*	BRA	dsw3
*------------
vlf5out:
dsw3:	LDHX	#vlf6		; next function
	STHX	vlfunction
	RTS
*==========================================================================
Inc_Mins:
* increment minutes
	ADD	time+1
	DAA
	STA	time+1
	
	CMP	#$60
	BLT	ims1

	SUB	#$60
	STA	time+1

* hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	ims1
	CLR	time

* zero the seconds
ims1:	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
* Decrement minutes
* (DAA doesn't work with SUB)
Dec_Mins:
	ADD	time+1
	DAA
	STA	time+1

	CMP	#$60
	BCS	dsms2

	SUB	#$40
	STA	time+1

* hours
dsms1:	LDA	time
	ADD	#$99		; subtract 1 hour
	DAA
	STA	time

	CMP	#$00
	BPL	dsms2
	MOV	#$23,time

* zero the seconds
dsms2:	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
Inc_Hrs:
* increment hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	ihs1
	CLR	time

* zero the seconds
ihs1:	
*	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
* Decrement hours
* (DAA doesn't work with SUB)
Dec_Hrs:
	LDA	time
	ADD	#$99		; subtract 1 hour
	DAA
	STA	time

	CMP	#$00
	BPL	dshs2
	MOV	#$23,time

* zero the seconds
dshs2:	
*	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*=============================================================================
* Get the character addresses ready
* Done during VBI
* Takes about 23 us (163 cycles)
Get_CH:	
	CLRH

* first character
	LDA	time		; convert to 12 hour time 
	CMP	#$12
	BLE	gchw1
	ADD	#$88		; add 100-12
	DAA

gchw1:	TSTA
	BNE	gch1a
	LDA	#$12		; 00 hours is 12 am 

gch1a:	PSHA
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch1		
	CLRA
gch1:	TAX

	LDA	CHTABLE,X
	STA	ch0adstor
	LDA	CHTABLE+1,X
	STA	ch0adstor+1

* second character
*	LDA	time
	PULA			; the corrected time
gch2a:	AND	#$0F
	ASLA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch2		
	CLRA
gch2:	TAX

	LDA	CHTABLE,X
	STA	ch1adstor
	LDA	CHTABLE+1,X
	STA	ch1adstor+1

* third character
	LDA	time+1
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch3		
	CLRA
gch3:	TAX

	LDA	CHTABLE,X
	STA	ch2adstor
	LDA	CHTABLE+1,X
	STA	ch2adstor+1

* fourth
	LDA	time+1
	AND	#$0F
	ASLA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch4		
	CLRA
gch4:	TAX

	LDA	CHTABLE,X
	STA	ch3adstor
	LDA	CHTABLE+1,X
	STA	ch3adstor+1

* fifth character
	LDA	time+2
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch5		
	CLRA
gch5:	TAX

	LDA	CHTABLE,X
	STA	ch4adstor
	LDA	CHTABLE+1,X
	STA	ch4adstor+1

* sixth character
	LDA	time+2
	AND	#$0F
	ASLA
	CMPA	#18		; we have already shifted the data x2
	BLE	gch6		
	CLRA
gch6:	TAX

	LDA	CHTABLE,X
	STA	ch5adstor
	LDA	CHTABLE+1,X
	STA	ch5adstor+1

* a or p
	LDA	time
	CMP	#$12
	BLT	gch7
	LDHX	#ch_p
	BRA	gch8
gch7:	LDHX	#ch_a
gch8	STHX	madstor

	RTS
*==========================================================================
Do_Time:
* Do Time

* field marker for the clockset function
	BSET	1,clockflag	
	
* If the time has just been set, reset the frame count
	BRCLR	0,clockflag,dt1	

	BCLR	0,clockflag
	MOV	#30,timecount		; 30 frames/sec

	LDHX	#TIMECAL	; restart the clock correction for 15.734 KHz
	STHX	clockcal

	BRA	dtout	

* frame count
dt1:	DEC	timecount
	BNE	dt2
	BSR	Inc_Secs
	MOV	#30,timecount	; 30 frames/sec
	
* Check if this is a correction cycle
dt2:	LDHX	clockcal
	AIX	#-1
	STHX	clockcal
	BNE	dtout		; nope
	LDHX	#TIMECAL	; restart the clock correction for 15.734 KHz
	STHX	clockcal

* Apply the correction
	DEC	timecount
	BNE	dtout

	BSR	Inc_Secs
	MOV	#30,timecount		; 30 frames/sec

dtout:	RTS
*==========================================================================
Inc_Secs:
* Increment the seconds
	LDA	time+2
	ADD	#1
	DAA
	STA	time+2
	
	CMP	#$60
	BNE	isout

	CLR	time+2
*----
* minutes
	LDA	time+1
	ADD	#1
	DAA
	STA	time+1
	
	CMP	#$60
	BNE	isout

	CLR	time+1

* hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	isout
	CLR	time

isout:	RTS
*==========================================================================
vlf6:
* ACC contains vline number
* ADC Data
* Set up characters for next line
*	BRA	vlf9out		; do characters?
	CMP	#160		; start characters at line 160
	BNE	vlf6out		; do characters?

	JSR	Setup1
	MOV	#$01,ADSCR	; AD Channel 1 is next

* Too much to do in one line
	LDHX	#vlf7			; start it and display it next time
	STHX	vlfunction

vlf6out:
	RTS
*-------------------------------------------------------------
Setup1:	MOV	#8,chline

* Read ADC
	MOV	ADR,bin		; read ADC
*	MOV	#123,bin	; for testing
	BSR	BIN_BCD		; Convert to BCD

* first digit
	LDA	bcd
	AND	#$0F
	ASLA
	CMPA	#18		; we have already shifted the data x2
	BLE	vlf6a		
	CLRA
vlf6a:	TAX

	CLRH
	LDA	CHTABLE,X
	STA	ch6adstor
	LDA	CHTABLE+1,X
	STA	ch6adstor+1

	LDHX	ch6adstor
	STHX	ch6address

	RTS
*-------------------------------------------------------------
vlf7:	JSR	Setup2

	LDHX	#vlf8			; start it and display it next time
	STHX	vlfunction

vlf7out:
	RTS
*-------------------------------------------------------------
Setup2:
* second digit
	LDA	bcd+1
	AND	#$F0
	LSRA
	LSRA
	LSRA
	CMPA	#18		; we have already shifted the data x2
	BLE	vlf7b	
	CLRA
vlf7b:	TAX

	CLRH
	LDA	CHTABLE,X
	STA	ch7adstor
	LDA	CHTABLE+1,X
	STA	ch7adstor+1

	LDHX	ch7adstor
	STHX	ch7address

* third digit
	LDA	bcd+1
	AND	#$0F
	ASLA
	CMPA	#18		; we have already shifted the data x2
	BLE	vlf7c		
	CLRA
vlf7c:	TAX

	CLRH
	LDA	CHTABLE,X
	STA	ch8adstor
	LDA	CHTABLE+1,X
	STA	ch8adstor+1

	LDHX	ch8adstor
	STHX	ch8address
*--------
	MOV	#4,chtimes

	RTS
*-------------------------------------------------------------
* Convert binary to bcd, 255. max
* uses ACC, Xreg, binl, bcd, bcd+1
* input from bin, output in bcd, bcd+1
BIN_BCD:
        CLR     bcd+1
        CLR     bcd
        LDX     #8
bcdloop:
        ROL     bin

	LDA	bcd+1
	ADC	bcd+1
	DAA
	STA	bcd+1

	LDA	bcd
	ADC	bcd
	DAA
	STA	bcd

        DBNZX   bcdloop

        RTS
*-------------------------------------------------------------
* Display the ADC characters
vlf8:	
*	BRA	vlf9		; do characters?

	LDA	chline
	BNE	vlf8a		; still going

	LDHX	#vlf9		; next group
	STHX	vlfunction
	BRA	vlf8out

vlf8a:	JSR	Display_ADC
vlf8out:
	RTS
*-------------------------------------------------------------
Display_ADC:
* Wait a little
	LDX	#30		; [2]
vlf8b:	DBNZX	vlf8b  		; [3]	

* do the characters
	LDHX	ch6address
	JSR	,X

	LDHX	ch7address
	JSR	,X

	LDHX	ch8address
	JSR	,X

	DEC	chtimes
	BNE	vlf8out		; repeat the line

	DEC	chline		; next line

	LDHX	ch6address
	AIX	#19		; a line takes 19 bytes
	STHX	ch6address

	LDHX	ch7address
	AIX	#19		; a line takes 19 bytes
	STHX	ch7address

	LDHX	ch8address
	AIX	#19		; a line takes 19 bytes
	STHX	ch8address

	MOV	#4,chtimes

	RTS
*===========================================================================
vlf9:
* ACC contains vline number
* ADC Data
* Set up characters for next line
*	BRA	vlf9out		; do characters?
	CMP	#210		; start characters at line 200
	BNE	vlf9out		; do characters?

	JSR	Setup1
	MOV	#$00,ADSCR	; AD Channel 0 is next

* Too much to do in one line
	LDHX	#vlf10		; start it and display it next time
	STHX	vlfunction

vlf9out:
	RTS
*-------------------------------------------------------------
vlf10:	JSR	Setup2

	LDHX	#vlf11		; start it and display it next time
	STHX	vlfunction

vlf10out:
	RTS
*-------------------------------------------------------------
* Display the ADC characters
vlf11:	
*	BRA	vlf9		; do characters?

	LDA	chline
	BNE	vlf11a		; still going

	LDHX	#vlf12		; next group
	STHX	vlfunction
	BRA	vlf8out

vlf11a:	JSR	Display_ADC
vlf11out:
	RTS
*-------------------------------------------------------------
* No more to do
vlf12:	RTS
*==========================================================================
	ORG	$F470
$INCLUDE 'chars.h'
*==========================================================================
******************************************************************************
* NOP Table
******************************************************************************
*==========================================================================
* Put NOPs here
	ORG	NOPORG
$INCLUDE 'nops1.h'   ; 456 NOPs (NOPSHI and NOPSLO must match this address)
*==========================================================================
******************************************************************************
* Vector Table
	ORG	ADCVEC		; A/D conversion complete		
	FDB	UNUSED

	ORG	KBVEC	 	; Keyyboard Vector			
	FDB	UNUSED

	ORG	TIM_OF_VEC  	; Timer Overflow Vector		
	FDB	TIM_INT

	ORG	TIM_CH1_VEC 	; Timer Channel 1 Vector		
	FDB	UNUSED

	ORG	TIM_CH0_VEC	; Timer Channel 2 Vector		
	FDB	UNUSED

	ORG	IRQ_VEC		; IRQ Vector				
	FDB	UNUSED

	ORG	SWI_VEC		; Software Interrupt			
	FDB	UNUSED

	ORG 	RESET_VEC
	FDB	START
*=============================================================================
